/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Class that maps an object to a <code>Collection</code> of objects.
 * 
 * @author Torgil
 * 
 */
public class HashBag<K, V> implements Serializable {
    
    public static final int DEFAULT_MAP_INITIAL_CAPACITY = 16;
    public static final int DEFAULT_COLLECTION_INITIAL_CAPACITY = 10;
    
    private Map<K, Collection<? extends V>>    m_hashBag;
    private int                                m_initialCollectionCapacity;
    private ICollectionFactory                 m_collectionFactory;
    
    /**
     * Creates a <code>HashBag</code> with default initial capabilities.
     *
     */
    public HashBag() {
        this(DEFAULT_MAP_INITIAL_CAPACITY, DEFAULT_COLLECTION_INITIAL_CAPACITY);
    }
    
    /**
     * Creates a HashBag with the specified internal capacity of the internal
     * <code>Map</code>.
     * 
     * @param initialHashCapacity
     */
    public HashBag(int initialHashCapacity) {
        this(initialHashCapacity, DEFAULT_COLLECTION_INITIAL_CAPACITY);
    }
    
    /**
     * Creates a HashBag with the specified initial capacities of the internal
     * <code>Map</code> and the mapped <code>Collections</code>.
     * 
     * @param initialHashCapacity
     * @param initialCollectionCapacity
     */
    public HashBag(int initialHashCapacity, int initialCollectionCapacity) {
        m_hashBag = new HashMap<K, Collection<? extends V>>(initialHashCapacity);
        m_initialCollectionCapacity = initialCollectionCapacity;
    }
    
    /**
     * Sets the <code>ICollectionFactory</code> that is used by this
     * <code>HashBag</code> when it needs to create a new value
     * <code>Collection</code> for a mapped key. The default behavoior is to
     * use a factory that creates <code>ArrayLists</code>. This method can be
     * used to customize the behavior of a <code>HashBag</code>, e.g. to
     * create a <code>HashBag</code> that only allows unique values mapped to
     * a key:
     * 
     * <pre>
     * HashBag bag = new HashBag();
     * bag.setCollectionFactory(ICollectionFactory.HASH_SET_FACTORY);
     * </pre>
     * 
     * @param f
     *            the factory to use for creating the value
     *            <code>Collection</code> when a new key is added to this
     *            <code>HashBag</code>. Passing <code>null</code> to this
     *            method results in the default behavior of creating
     *            <code>ArrayLists</code>.
     */
    public void setCollectionFactory(ICollectionFactory f) {
        m_collectionFactory = f;
    }
    
    /**
     * Adds the specified value to this <code>HashBag</code>.
     * 
     * @param key
     *            the key to which the value is mapped.
     * @param value
     *            the value to add.
     */
    @SuppressWarnings(value={"unchecked"})
    public void put(K key, V value) {
        Object o = m_hashBag.get(key);
        if (o instanceof Collection) {
            ((Collection) o).add(value);
        }
        else {
            Collection<V> list = createValueCollection();
            list.add(value);
            m_hashBag.put(key, list);
        }
    }
    
    /**
     * Puts the specified values in this <code>HashBag</code>.
     * 
     * @param key
     *            the key to which the values are mapped.
     * @param value
     *            a <code>Collection</code> containing the values to add.
     */
    public void putValues(K key, Collection<? extends V> value) {
        m_hashBag.put(key, value);
    }
    
    /**
     * Removes all values that are mapped to the specified key.
     * 
     */
    public void remove(K key) {
        m_hashBag.remove(key);
    }
    
    /**
     * Removes a value from this <code>HashBag</code>.
     * 
     * @param key
     *            the key to which the value is mapped.
     * @param value
     *            the value to remove.
     */
    public void remove(K key, V value) {
        Collection<? extends V> list = m_hashBag.get(key);
        if (list != null) {
            list.remove(value);
            if (list.isEmpty()) {
                m_hashBag.remove(key);
            }
        }
    }
    
    /**
     * Returns the values that are currently stored in this <code>HashBag</code>
     * for the specified key.
     * 
     * @param key
     *            the key for which to retrieve the stored values.
     * @return a <code>List</code> with the values that are currently mapped
     *         to the specified key. This method never returns <code>null</code>;
     *         an empty list is returned if nothing is mapped to the specified
     *         key.
     */
    public List<V> get(K key) {
        Collection<? extends V> values = m_hashBag.get(key);
        return (values != null) ? new ArrayList<V>(values) : new ArrayList<V>();
    }
    
    /**
     * Returns the keys in this <code>HashBag</code> in the form of a
     * <code>Set</code>.
     * 
     * @return a <code>Set</code> containing the current keys in this
     *         <code>HashBag</code>. An empty <code>Set</code> is returned
     *         if this bag is empty.
     * 
     */
    public Set<K> keySet() {
        return m_hashBag.keySet();
    }
    
    /**
     * Returns the entries in this <code>HashBag</code> in the form of a
     * <code>Map.Entry</code>.
     * 
     */
    public Set<Map.Entry<K, Collection<? extends V>>> entrySet() {
        return m_hashBag.entrySet();
    }
    
    /**
     * Returns the number of keys in the <code>HashBag</code>.
     * 
     */
    public int size() {
        return m_hashBag.size();
    }

    /**
     * Checks if this <code>HashBag</code> contains the specified key.
     * 
     */
    public boolean contains(K key) {
        return m_hashBag.containsKey(key);
    }

    /**
     * Checks if this <code>HashBag</code> contains the specified key-value
     * pair.
     * 
     */
    public boolean contains(K key, V value) {
        if (m_hashBag.containsKey(key)) {
            Collection<? extends V> values = m_hashBag.get(key);
            return values.contains(value);
        }
        return false;
    }
    
    /**
     * Removes all entries from this <code>HashBag</code>.
     *
     */
    public void clear() {
        m_hashBag.clear();
    }
    
    private Collection<V> createValueCollection() {
        if (m_collectionFactory == null) {
            m_collectionFactory = ICollectionFactory.ARRAY_LIST_FACTORY;
        }
        return m_collectionFactory.newCollection(m_initialCollectionCapacity);
    }
}
