/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 * Helper class which provides some list manipulation routines.
 * 
 * @author Torgil
 * @author EmanuelN
 */
public class KongaListUtils
{
	private KongaListUtils() {
		// No instances of this class.
	}
    
	/**
     * Adds the Object <code>o</code> to the List <code>list</code>
     * 
     * @param o
     *            Object to add to the list.
     * @param list
     *            List to add object to.
     * @return <code>true</code> if the object was added to the list,
     *         otherwise <code>false</code>.
     */
	public static <S, T extends S> boolean addUniqueElement(T o, List<S> list) {
		if ( o != null && list != null ) {
			if ( !list.contains(o) ) {
				list.add(o);
				return true;
			}
		}
		return false;
	}

    /**
     * Creates and returns a <tt>List</tt> of the specified items.
     * 
     * @param items
     */
    public static <T> List<T> asList(T... items) {
        return new ArrayList<T>(Arrays.asList(items));
    }

    /**
     * Creates and returns a <tt>List</tt> of the items returned by the
     * specified iterator. An empty list is returned if
     * <code>items.hasNext() == false</code>.
     * 
     * @throws NullPointerException
     *             if <code>items</code> is null.
     * @since 1.2
     */
    public static <T> List<T> asList(Iterator<T> items) {
        List<T> list = new ArrayList<T>();
        for ( ; items.hasNext(); ) {
            list.add(items.next());
        }
        return list;
    }

    /**
     * Creates and returns a <tt>List</tt> from an <tt>Iterable</tt>.
     * 
     * @param items
     * @throws NullPointerException
     *             if <tt>items</tt> is null.
     */
    public static <T> List<T> asList(Iterable<T> items) {
        List<T> list = new ArrayList<T>();
        for (T item : items) {
            list.add(item);
        }
        return list;
    }
    
	/**
     * Adds all elements from <tt>newElements</tt> if not already present in
     * <tt>originalList</tt>.
     * 
     * @param newElements
     *            Elements to add.
     * @param list
     *            The list where unique elements are to be added.
     */
	public static <S, T extends S> void addUniqueElements(List<T> newElements, List<S> list) {		
		addLists(newElements, list, false);
	}

	/**
	 * Adds the elements of the List <code>added</code> to the List <code>original</code>
	 * 
	 * @param newElements
	 * @param list
	 * @param bAddDuplicates
	 */	
	public static <S, T extends S> void addLists(List<T> newElements, List<S> list, boolean bAddDuplicates)
	{
		if( list == null || newElements == null )
			return;
		
		if( bAddDuplicates ) {
			list.addAll(newElements);
		}
		else {
		    for( T t : newElements ) {
		        if( !list.contains(t) )
		            list.add(t);
		    }
		}
	}	
	
	/**
	 * Adds all the objects from the specified <tt>Array</tt> to the 
	 * specified <tt>List</tt>.
	 * 
	 * @param array
	 * @param list
	 */
	public static <S, T extends S> void addFromArray(T[] array, List<S> list) {
		if (list == null)
			return;
		if (array == null)
			return;
		for (int i = 0; i < array.length; ++i)
			list.add(array[i]);
	}
	
	
	public static String toString(List list) {
		return toString(list, ", ");
	}
	
	public static String toString(List<?> list, String sSeparator) {
        String s = null;
		if( list == null ) {
			s = "<null>";
		}
		else {
			StringBuffer sb = new StringBuffer("{ ");
			int n = 0;
			for( Iterator<?> it = list.iterator(); it.hasNext(); ) {
				if( n > 0 )
					sb.append(sSeparator);
				sb.append(it.next());
				++n;
			}
			sb.append("}");
			s = sb.toString();
		}
        return s;
	}
    
}
