/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jitterbit.util.collections.HashBag;
import org.jitterbit.util.lang.KongaObjectUtils;

/**
 * Maps namespace prefixes to namespace URIs.
 * 
 * @author Torgil
 * 
 * @since 1.2
 *
 */
public class NamespacePrefixMap {

    private final HashBag<String, String>   m_uriToPrefix;
    private final Map<String, String>       m_prefixToUri;
    private String                          m_defaultNamespace;
    
    public NamespacePrefixMap() {
        m_uriToPrefix = new HashBag<String, String>();
        m_prefixToUri = new HashMap<String, String>();
    }
    
    /**
     * Adds a mapping between the specified prefix and the specified URI.
     * 
     * @param prefix
     *            the prefix representing the URI. If this argument is
     *            <code>null</code> or an empty string, the URI is regarded as
     *            the default namespace.
     * @param uri
     *            the namespace URI to be mapped to <code>prefix</code>.
     * @throws IllegalArgumentException
     *             if <code>prefix</code> is a non-blank string, and
     *             <code>uri</code> is <code>null</code> or
     *             <code>blank</code>.
     */
    public void add(String prefix, String uri) {
        boolean emptyPrefix = false;
        if ((prefix == null) || (prefix.length() == 0)) {
            prefix = "";
            m_defaultNamespace = uri;
            emptyPrefix = true;
        }
        if ((uri != null) && (uri.length() > 0)) {
            m_prefixToUri.put(prefix, uri);
            m_uriToPrefix.put(uri, prefix);
        }
        else if (!emptyPrefix) {
            throw new IllegalArgumentException("A prefix cannot be mapped to an empty " +
                    "namespace URI.");
        }
    }
    
    /**
     * Returns the namespace URI that is mapped to the specified prefix.
     * 
     * @param prefix
     *            the prefix, the URI of which to retrieve.
     * @return a String holding the namespace URI that has been mapped to
     *         <code>prefix</code>. <code>null</code> is returned if the
     *         specified prefix has not been mapped.
     */
    public String getNamespaceURI(String prefix) {
        return m_prefixToUri.get(prefix);
    }
    
    /**
     * Returns all prefixes that are currently mapped to the specified namespace
     * URI.
     * 
     * @param uri
     *            the URI the prefixes of which to return
     * @return a <code>List</code> of the prefixes that are mapped to the
     *         specified URI. An empty list is returned if no prefixes are
     *         mapped to the URI.
     */
    public List<String> getPrefixes(String uri) {
        return m_uriToPrefix.get(uri);
    }
    
    /**
     * Checks if the specified URI is the default namespace.
     * 
     */
    public boolean isDefaultNamespace(String uri) {
        return KongaObjectUtils.areEqual(uri, m_defaultNamespace);
    }
    
    /**
     * Returns the default namespace.
     * 
     * @return a String holding the default namespace URI. <code>null</code>
     *         is returned if no default namespace has been specified.
     */
    public String getDefaultNamespace() {
        return m_defaultNamespace;
    }
    
    /**
     * Expands the specified <code>IQualifiedName</code> to an
     * <code>IExpandedName</code>, using the namespace information in this
     * map.
     * 
     * @param name
     *            the <code>IQualifiedName</code> to expand.
     * @param nameFactory
     *            an <code>XmlNameFactory</code> that will be asked to create
     *            the requested <code>IExpandedName</code> instance.
     * @return an <code>IExpandedName</code> instance corresponding to the
     *         specified qualified name. The prefix part of the qualified name
     *         is translated into a namespace URI using the mapped namespaces in
     *         this map.
     * @throws NullPointerException
     *             if <code>name</code> or <code>nameFactory</code> is
     *             <code>null</code>.
     * @throws IllegalArgumentException
     *             if <code>name</code> cannot be expanded into a valid
     *             <code>IExpandedName</code> by the specified name factory.
     */
    public IExpandedName expand(IQualifiedName name, XmlNameFactory nameFactory) {
        String prefix = name.getPrefix();
        String uri = null;
        if (prefix.length() == 0) {
            uri = m_defaultNamespace;
        }
        else {
            uri = getNamespaceURI(prefix);
        }
        return nameFactory.newExpandedName(uri, name.getLocalName());
    }
    
    /**
     * Collapses the specified <code>IExpandedName</code> to an
     * <code>IQualifiedName</code>, using the namespace information in this
     * map.
     * 
     * @param name
     *            the <code>IExpandedName</code> to collapse.
     * @param nameFactory
     *            an <code>XmlNameFactory</code> that will be asked to create
     *            the requested <code>IQualifiedName</code> instance.
     * @return an <code>IQualifiedName</code> instance corresponding to the
     *         specified qualified name. The namespace URI of the expanded name
     *         is translated into a namespace prefix using the mapped namespaces
     *         in this map. If more than one prefix has been mapped to the URI,
     *         the first one encountered in the map is used. If the URI is the
     *         default namespace, or if no prefix has been mapped to the URI,
     *         the returned <code>IQualifiedName</code> will be unprefixed.
     * @throws NullPointerException
     *             if <code>name</code> or <code>nameFactory</code> is
     *             <code>null</code>.
     * @throws IllegalArgumentException
     *             if <code>name</code> cannot be expanded into a valid
     *             <code>IQualifiedName</code> by the specified name factory.
     */
    public IQualifiedName collapse(IExpandedName name, XmlNameFactory nameFactory) {
        String uri = name.getNamespace();
        String prefix = getPrefix(uri);
        return nameFactory.newQualifiedName(prefix, name.getLocalName());
    }
    
    private String getPrefix(String uri) {
        if ((uri.length() == 0) || uri.equals(m_defaultNamespace)) {
            return "";
        }
        return aPrefix(uri);
    }
    
    private String aPrefix(String uri) {
        List<? extends String> available = m_uriToPrefix.get(uri);
        return available.isEmpty() ? "" : available.get(0);
    }
}
