/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.io.File;
import java.io.IOException;
import java.io.Reader;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.TypeInfo;
import org.w3c.dom.UserDataHandler;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * {@link RootNodeExtractor} implementation that uses a SAX parser to extract
 * the root node.
 * 
 * @author Torgil
 * 
 * @since 1.2
 *
 */
/*package-private*/
final class SaxRootNodeExtractor extends RootNodeExtractor {

    @Override
    public Element extractRoot(File xmlFile) throws KongaXmlException {
        try {
            SAXParser parser = getSaxParser();
            Handler handler = new Handler();
            parser.parse(xmlFile, handler);
            return handler.getRoot();
        }
        catch (SAXException ex) {
            throw new KongaXmlException("Failed to parse the specified XML file.", ex);
        }
        catch (IOException ex) {
            throw new KongaXmlException("Failed to read the specified XML file.", ex);
        }
    }

    @Override
    public Element extractRoot(Reader reader) throws KongaXmlException {
        try {
            SAXParser parser = getSaxParser();
            Handler handler = new Handler();
            InputSource is = new InputSource(reader);
            parser.parse(is, handler);
            return handler.getRoot();
        }
        catch (SAXException ex) {
            throw new KongaXmlException("Failed to parse the specified XML.", ex);
        }
        catch (IOException ex) {
            throw new KongaXmlException("Failed to read the specified XML.", ex);
        }
    }

    private SAXParser getSaxParser() throws KongaXmlException {
        try {
            SAXParserFactory factory = getParserFactory();
            return factory.newSAXParser();
        }
        catch (Exception ex) {
            throw new KongaXmlException("Failed to create the SAX parser.", ex);
        }
    }
    
    private SAXParserFactory getParserFactory() {
        SAXParserFactory f = SAXParserFactory.newInstance();
        f.setNamespaceAware(true);
        return f;
    }

    
    private static final class Handler extends DefaultHandler {

        private Element m_root;
        
        @Override
        public void startElement(String uri, 
                                 String localName, 
                                 String qName, 
                                 Attributes attributes) throws SAXException {
            if (m_root == null) {
                m_root = new ElementImpl(uri, qName, localName, attributes);
            }
        }
        
        public Element getRoot() {
            return m_root;
        }
    }
    

    private static final class ElementImpl implements Element {

        private final String        m_namespaceUri;
        private final String        m_tagName;
        private final String        m_prefix;
        private final String        m_localName;
        private final Attributes    m_attributes;
        
        public ElementImpl(String namespaceUri, 
                           String qName,
                           String localName, 
                           Attributes attrs) {
            m_namespaceUri = namespaceUri;
            m_tagName = qName;
            int sep = qName.lastIndexOf(':');
            m_prefix = (sep >= 0) ? qName.substring(0, sep) : "";
            m_localName = localName;
            m_attributes = attrs;
        }
        
        public String getAttribute(String name) {
            return m_attributes.getValue(name);
        }

        public boolean hasAttribute(String name) {
            return m_attributes.getValue(name) != null;
        }

        public String getLocalName() {
            return m_localName;
        }

        public String getNamespaceURI() {
            return m_namespaceUri;
        }

        public boolean hasAttributes() {
            return m_attributes.getLength() > 0;
        }

        public short getNodeType() {
            return Node.ELEMENT_NODE;
        }

        public String getPrefix() {
            return m_prefix;
        }

        public String getTagName() {
            return m_tagName;
        }

        public String getNodeName() {
            return m_tagName;
        }

        public String getNodeValue() {
            // Not supported.
            return null;
        }

        public Attr getAttributeNode(String name) {
            // Not supported.
            return null;
        }

        public Attr getAttributeNodeNS(String namespaceURI, String localName) {
            // Not supported.
            return null;
        }

        public String getAttributeNS(String namespaceURI, String localName) {
            // Not supported.
            return null;
        }

        public NodeList getElementsByTagName(String name) {
            // Not supported.
            return null;
        }

        public NodeList getElementsByTagNameNS(String namespaceURI, String localName) {
            // Not supported.
            return null;
        }

        public TypeInfo getSchemaTypeInfo() {
            // Not supported.
            return null;
        }

        public boolean hasAttributeNS(String namespaceURI, String localName) {
            // Not supported.
            return false;
        }

        public void removeAttribute(String name) {
            // Not supported.
        }

        public Attr removeAttributeNode(Attr oldAttr) {
            // Not supported.
            return null;
        }

        public void removeAttributeNS(String namespaceURI, String localName) {
            // Not supported.
        }

        public void setAttribute(String name, String value) {
            // Not supported.
        }

        public Attr setAttributeNode(Attr newAttr) {
            // Not supported.
            return null;
        }

        public Attr setAttributeNodeNS(Attr newAttr) {
            // Not supported.
            return null;
        }

        public void setAttributeNS(String namespaceURI, String qualifiedName, String value) {
            // Not supported.
        }

        public void setIdAttribute(String name, boolean isId) {
            // Not supported.
        }

        public void setIdAttributeNode(Attr idAttr, boolean isId) {
            // Not supported.
        }

        public void setIdAttributeNS(String namespaceURI, String localName, boolean isId) {
            // Not supported.
        }

        public Node appendChild(Node newChild) {
            // Not supported.
            return null;
        }

        public Node cloneNode(boolean deep) {
            // Not supported.
            return null;
        }

        public short compareDocumentPosition(Node other) {
            // Not supported.
            return 0;
        }

        public NamedNodeMap getAttributes() {
            // Not supported.
            return null;
        }

        public String getBaseURI() {
            // Not supported.
            return null;
        }

        public NodeList getChildNodes() {
            // Not supported.
            return null;
        }

        public Object getFeature(String feature, String version) {
            // Not supported.
            return null;
        }

        public Node getFirstChild() {
            // Not supported.
            return null;
        }

        public Node getLastChild() {
            // Not supported.
            return null;
        }

        public Node getNextSibling() {
            // Not supported.
            return null;
        }

        public Document getOwnerDocument() {
            // Not supported.
            return null;
        }

        public Node getParentNode() {
            // Not supported.
            return null;
        }

        public Node getPreviousSibling() {
            // Not supported.
            return null;
        }

        public String getTextContent() {
            // Not supported.
            return null;
        }

        public Object getUserData(String key) {
            // Not supported.
            return null;
        }

        public boolean hasChildNodes() {
            // Not supported.
            return false;
        }

        public Node insertBefore(Node newChild, Node refChild) {
            // Not supported.
            return null;
        }

        public boolean isDefaultNamespace(String namespaceURI) {
            // Not supported.
            return false;
        }

        public boolean isEqualNode(Node arg) {
            // Not supported.
            return false;
        }

        public boolean isSameNode(Node other) {
            // Not supported.
            return false;
        }

        public boolean isSupported(String feature, String version) {
            // Not supported.
            return false;
        }

        public String lookupNamespaceURI(String prefix) {
            // Not supported.
            return null;
        }

        public String lookupPrefix(String namespaceURI) {
            // Not supported.
            return null;
        }

        public void normalize() {
            // Not supported.
        }

        public Node removeChild(Node oldChild) {
            // Not supported.
            return null;
        }

        public Node replaceChild(Node newChild, Node oldChild) {
            // Not supported.
            return null;
        }

        public void setNodeValue(String nodeValue) {
            // Not supported.
        }

        public void setPrefix(String prefix) {
            // Not supported.
        }

        public void setTextContent(String textContent) {
            // Not supported.
        }

        public Object setUserData(String key, Object data, UserDataHandler handler) {
            // Not supported.
            return null;
        }
    }
}
