/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Stack;
import java.util.TreeSet;

import org.apache.commons.lang.StringEscapeUtils;
import org.jitterbit.util.collections.Pair;

/**
 * Construct XML string contents. Note that this class does not offer any
 * XML processing or manipulation functionality.
 * 
 * @author Torgil
 * 
 */
public class XmlStringBuilder {

    private static final String INDENT = "    ";
    private static final String LINE_BREAK = System.getProperty("line.separator");
    
    private Element         m_root;
    private Stack<Element>  m_elements;
    private String          m_processingInstruction;
    
    public XmlStringBuilder() {
        m_elements = new Stack<Element>();
        m_processingInstruction = KongaXmlUtils.UTF_8_PROCESSING_INSTRUCTION;
    }
    
    /**
     * Sets the processing instruction that will be added at the top of the
     * produced XML. The default is {@link KongaXmlUtils#UTF_8_PROCESSING_INSTRUCTION}.
     * 
     */
    public void setProcessingInstruction(String s) {
        m_processingInstruction = s;
    }
    
    /**
     * Adds a new element to this builder. The added element will be the child
     * of the current element, or the XML root if it is the first element to be
     * added.
     * <p>
     * All opened elements must be closed, by calling {@link #closeElement()}.
     * 
     * @param name
     *            the name of the element to add.
     * @throws XmlStringBuilderException
     *             if the element is added after the root element has already
     *             been closed.
     */
    public void addElement(String name) {
        Element e = new Element(name);
        if (m_elements.isEmpty() && m_root != null) {
            throw new XmlStringBuilderException("Illegal XML: only one root is allowed.");
        }
        if (m_root == null) {
            m_root = e;
        }
        if (!m_elements.isEmpty()) {
            m_elements.peek().addChild(e);
        }
        m_elements.add(e);
    }
    
    /**
     * Closes the current element. Once an element has been closed it cannot be
     * changed, i.e. it is impossible to add more attributes or child elements
     * to it. When an element is closed, its parent element becomes the current
     * element.
     * <p>
     * When the root element has been closed, no more elements or attributes can
     * be added to this builder.
     * 
     * @throws XmlStringBuilderException
     *             if there is no open element.
     */
    public void closeElement() {
        if (m_elements.isEmpty()) {
            throw new XmlStringBuilderException("There is no element to close.");
        }
        m_elements.pop();
    }

    /**
     * Closes all open elements, including the root element.
     * <p>
     * <strong>Note: </strong> Once this method has been called, the present XML
     * represented by this builder can no longer be modified.
     * 
     * @throws XmlStringBuilderException
     *             if there are no open elements.
     * @since 1.2             
     */
    public void closeAllElements() {
        if (m_elements.isEmpty()) {
            throw new XmlStringBuilderException("There are no elements to close.");
        }
        m_elements.removeAllElements();
    }
    
    /**
     * Adds an attribute to the current element.
     * 
     * @throws XmlStringBuilderException
     *             if there is no open element.
     * @throws IllegalArgumentException
     *             if <tt>name</tt> is null.
     */
    public void addAttribute(String name, String value) {
        if (name == null) {
            throw new IllegalArgumentException("name can't be null.");
        }
        if (value == null) {
            value = "";
        }
        addAttribute(new Pair<String, String>(name, value));
    }
    
    /**
     * Adds an attribute to the current element.
     * 
     * @throws XmlStringBuilderException
     *             if there is no open element.
     * @throws IllegalArgumentException
     *             if <tt>attribute</tt> is null.
     */
    public void addAttribute(Pair<String, String> attribute) {
        if (attribute == null) {
            throw new IllegalArgumentException("attribute can't be null.");
        }
        if (m_elements.isEmpty()) {
            throw new XmlStringBuilderException("There is no element to add an attribute to.");
        }
        m_elements.peek().addAttribute(attribute);
    }
    
    /**
     * Sets the value of the current element.
     * 
     * @throws XmlStringBuilderException
     *             if there is no open element.
     */
    public void setElementValue(String value) {
        if (m_elements.isEmpty()) {
            throw new XmlStringBuilderException("There is no element to set a value for.");
        }
        m_elements.peek().setValue(value);
    }
    
    /**
     * Retrieves the XML string produced by this builder.
     * 
     * @throws XmlStringBuilderException
     *             if the current state cannot produce valid XML, for instance
     *             if there are any elements left open.
     * 
     */
    public String getXml() {
        if (!m_elements.isEmpty()) {
            throw new XmlStringBuilderException("There are still open elements.");
        }
        StringBuilder sb = new StringBuilder();
        if (m_processingInstruction != null) {
            sb.append(m_processingInstruction);
        }
        sb.append(LINE_BREAK);
        if (m_root != null) {
            sb.append(m_root.getXml(""));
        }
        return sb.toString();
    }
    
    
    private static class Element {
        
        private String                      m_name;
        private String                      m_value;
        private Set<Pair<String, String>>   m_attributes;
        private List<Element>               m_children;
        
        public Element(String name) {
            m_name = name;
        }
        
        public void setValue(String value) {
            m_value = value;
        }
        
        public void addAttribute(Pair<String, String> attribute) {
            if (m_attributes == null) {
                m_attributes = new TreeSet<Pair<String,String>>(
                                Pair.<String, String>compareFirst());
            }
            m_attributes.add(attribute);
        }
        
        public void addChild(Element element) {
            if (m_children == null) {
                m_children = new ArrayList<Element>();
            }
            m_children.add(element);
        }

        public String getXml(String indent) {
            StringBuilder sb = new StringBuilder(indent);
            sb.append("<").append(m_name);
            appendAttributes(sb);
            if (hasContent()) {
                sb.append(">");
                appendValue(sb);
                appendChildren(sb, indent);
                if (hasChildren()) {
                    sb.append(indent);
                }
                sb.append("</").append(m_name).append(">");
            }
            else {
                sb.append(" />");
            }
            sb.append(LINE_BREAK);
            return sb.toString();
        }
        
        private void appendAttributes(StringBuilder sb) {
            if (m_attributes == null) {
                return;
            }
            for (Pair<String, String> attribute : m_attributes) {
                sb.append(" ").append(attribute.getFirst()).append("=\"");
                sb.append(StringEscapeUtils.escapeXml(attribute.getSecond())).append("\"");
            }
        }
        
        private boolean hasContent() {
            return (m_value != null) || (m_children != null);
        }
        
        private boolean hasChildren() {
            return m_children != null;
        }
        
        private void appendValue(StringBuilder sb) {
            if (m_value != null) {
                sb.append(StringEscapeUtils.escapeXml(m_value));
            }
        }
        
        private void appendChildren(StringBuilder sb, String indent) {
            if (m_children == null) {
                return;
            }
            String childIndent = indent + INDENT;
            sb.append(LINE_BREAK);
            for (Element child : m_children) {
                sb.append(child.getXml(childIndent));
            }
        }
    }
}
