/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml.sax;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;
import java.util.Stack;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Default implementation of an <tt>AbstractKongaSaxParser</tt>.
 * 
 * @author Torgil
 *
 */
public class DefaultKongaSaxParser extends AbstractKongaSaxParser {

    private Map<String, IStartElementHandlerFactory> m_startElementHandlerFactories =
        new HashMap<String, IStartElementHandlerFactory>();
    private Map<String, IEndElementHandlerFactory> m_endElementHandlerFactories =
        new HashMap<String, IEndElementHandlerFactory>();
    
    private Stack<IStartElementHandler>    m_startElementHandlers =
        new Stack<IStartElementHandler>();
    private Stack<IEndElementHandler>    m_endElementHandlers =
        new Stack<IEndElementHandler>();


    /**
     * 
     * @param file
     * @throws KongaSaxParserException
     */
    public void load(File file) throws KongaSaxParserException {
        InputStream is = null;
        try {
            load(new FileInputStream(file));
        }
        catch (FileNotFoundException e) {
            throw new KongaSaxParserException(e);
        }
        catch (FactoryConfigurationError e) {
            Exception cause = new Exception(e);
            throw new KongaSaxParserException(cause);
        }
        finally {
            close(is);
        }
    }
    
    /**
     * 
     * @param resource
     * @throws KongaSaxParserException
     */
    public void load(String resource) throws KongaSaxParserException {
        InputStream is = null;
        try {
            load(getClass().getClassLoader().getResourceAsStream(resource));
        }
        catch (FactoryConfigurationError e) {
            Exception cause = new Exception(e);
            throw new KongaSaxParserException(cause);
        }
        finally {
            close(is);
        }
    }
    
    /**
     * 
     * @param is
     * @throws KongaSaxParserException
     */
    public void load(InputStream is) throws KongaSaxParserException {
        SAXParserFactory endElementFactory = SAXParserFactory.newInstance();
        endElementFactory.setNamespaceAware(true);
        try {
            SAXParser parser = endElementFactory.newSAXParser();
            if (parser != null) {
                parser.parse(is, this);
            }
        }
        catch (ParserConfigurationException e) {
            throw new KongaSaxParserException(e);
        }
        catch (SAXException e) {
            throw new KongaSaxParserException(e);
        }
        catch (IOException e) {
            throw new KongaSaxParserException(e);
        }
    }
    
    /**
     * 
     * @param reader
     * @throws KongaSaxParserException
     * @since 1.2
     */
    public void load(Reader reader) throws KongaSaxParserException {
        SAXParserFactory endElementFactory = SAXParserFactory.newInstance();
        endElementFactory.setNamespaceAware(true);
        try {
            SAXParser parser = endElementFactory.newSAXParser();
            if (parser != null) {
                InputSource is = new InputSource(reader);
                parser.parse(is, this);
            }
        }
        catch (ParserConfigurationException e) {
            throw new KongaSaxParserException(e);
        }
        catch (SAXException e) {
            throw new KongaSaxParserException(e);
        }
        catch (IOException e) {
            throw new KongaSaxParserException(e);
        }
    }
    
    /**
     * 
     * @param name
     * @param factory
     */
    public final void addStartElementHandlerFactory(String name, IStartElementHandlerFactory factory) {
        m_startElementHandlerFactories.put(name, factory);
    }
    
    /**
     * 
     * @param name
     * @param factory
     */
    public final void addEndElementHandlerFactory(String name, IEndElementHandlerFactory factory) {
        m_endElementHandlerFactories.put(name, factory);
    }
    
    /**
     * 
     * @param handler
     */
    public final void pushStartElementHandler(IStartElementHandler handler) {
        m_startElementHandlers.push(handler);
    }

    public final void popStartElementHandler() {
        m_startElementHandlers.pop();
    }

    /**
     * 
     * @param handler
     */
    public final void pushEndElementHandler(IEndElementHandler handler) {
        m_endElementHandlers.push(handler);
    }

    public final void popEndElementHandler() {
        m_endElementHandlers.pop();
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        super.startElement(uri, localName, qName, attributes);
        IStartElementHandlerFactory startElementFactory = 
                            m_startElementHandlerFactories.get(localName);
        if (startElementFactory != null) {
            m_startElementHandlers.push(startElementFactory.getHandler());
        }
        IEndElementHandlerFactory endElementFactory = 
                            m_endElementHandlerFactories.get(localName);
        if (endElementFactory != null) {
            m_endElementHandlers.push(endElementFactory.getHandler());
        }
        if (!m_startElementHandlers.empty()) {
            m_startElementHandlers.peek().handleStart(this);
        }
    }
    
    @Override
    public void endElement(String uri, String localName, String qName)
                    throws SAXException {
        // If there is a current EndElementHandler we will give it a chance
        // to run first:
        if (!m_endElementHandlers.empty()) {
            m_endElementHandlers.peek().handleEnd(this);
        }

        // If there is a Start- or an EndElementHandler registered for this
        // element it was pushed to the stack in startElement. Now it's time
        // to remove it from the stack:
        if (m_startElementHandlerFactories.containsKey(localName)) {
            m_startElementHandlers.pop();
        }
        if (m_endElementHandlerFactories.containsKey(localName)) {
            m_endElementHandlers.pop();
        }

        super.endElement(uri, localName, qName);
    }

    // We do not use KongaIoUtils.close(is) since we want to keep the XML
    // library as lightweight as possible.
    private static void close(InputStream is) {
        if (is != null) {
            try {
                is.close();
            }
            catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }
}
